require 'andand'

class TrailsController < ApplicationController
  before_action :set_trail, only: [
    :show, :match_guideposts, :update_kilometraz, :check,
    :edit, :edit_with_kct, :edit_with_osm, :edit_with_osm_and_kct,
    :update, :destroy, :update_with_kct
  ]
  before_action :ensure_admin!
  
  def index
    @trails = Trail.all
    
    # filter
    if params[:search]
      @trails = @trails.where("trails.kctref LIKE '%#{params[:search]}%'")
    end
    
    # sort
    @trails = if params[:sort] == 'priority'
      @trails.joins(:kct_trail).order('kct_trails.priority IS NULL').order('kct_trails.priority').order(:kctref)
    else
      @trails.order(:kctref)
    end
    
    # paginate
    @trails = @trails.page(params[:page])
  end
  
  def show
    @osm_likely_trails = OsmTrail.all_by_trail(@trail)
  end
  
  def new
    @trail = Trail.new(trail_params)
    create if params[:doit]
  end
  
  def check                # provede nejruznejsi kontroly konzistence cesty
  end
  
  def edit
    @trail.assign_attributes(trail_params) if trail_params
    update if params[:doit]
    @osm_trail = @trail.osm_trail
  end
  
  def edit_with_osm_and_kct                                       
    @merge_score_treshold = 200
    @guideposts = @trail.guideposts.to_a
    @kct_signs = @trail.kct_trail.kct_signs.sort{|a,b| a.km<=>b.km}
    @osm_signs_and_guideposts = @trail.osm_trail.andand.osm_signs.to_a 
        
    @osm_signs_and_guideposts += 
      @trail.kct_trail.kct_guideposts.collect{|kg| kg.osm_guidepost}.compact -          # guideposty které jsou už spárované odjindy
      @osm_signs_and_guideposts.collect{|os| os.osm_guidepost}           # odebrat ty co už tam jsou dokonce jako signs
    
    unused_kct_signs = @kct_signs.reject{|kct_sign| kct_sign.exists? }
#     raise @osm_signs_and_guideposts.collect{|os| [os.osm_guidepost.nazev, os.kctref]}.inspect
    
    # vyrobíme hash @estimated_relations - postupně k osm_sign hledáme pravděpodobné kct_sign
    @estimated_relations = {}

    @osm_signs_and_guideposts.each do |osm_sign_or_guidepost|
      if osm_sign_or_guidepost.exists?
        if osm_sign_or_guidepost.is_a?(OsmSign)
          estimated_kct_sign = osm_sign_or_guidepost.kct_sign
#           raise [osm_sign_or_guidepost.osm_guidepost.nazev, osm_sign_or_guidepost.kctref,estimated_kct_sign.kct_trail].inspect
        else
          estimated_kct_sign = KctKilometraz.find_by(:kct_trail => @trail.kct_trail, :kct_guidepost => osm_sign_or_guidepost.kct_guidepost)
        end
      else  # pripad ze OsmSign nema prislusny Sign (např proto že nemá guidepost)
        max_pair = unused_kct_signs.inject([0,nil]) do |amax_pair,kct_sign|
          if amax_pair[0] < Sign.merge_score(kct_sign,osm_sign_or_guidepost)
            [Sign.merge_score(kct_sign,osm_sign_or_guidepost),kct_sign]
          else
            amax_pair
          end
        end
        estimated_kct_sign = max_pair[1]
      end
      @estimated_relations[estimated_kct_sign] = [] if @estimated_relations[estimated_kct_sign].nil?
      @estimated_relations[estimated_kct_sign] << osm_sign_or_guidepost
    end

    # teď ke kčt_sign kde ještě nic není zkusíme najít guidepost volně v terénu podle geografické polohy
    unused_kct_signs = unused_kct_signs.select{|kct_sign| @estimated_relations[kct_sign].nil? }
    @other_guideposts = unused_kct_signs.collect do |kct_sign|
      estimated_km = kct_sign.estimated_km_from_osm_kms
      @trail.osm_trail.find_guideposts_near_position(estimated_km)
    end.flatten.uniq.reject{ |g| @guideposts.include?(g) }

    @other_guideposts.each do |osm_guidepost|
      max_pair = unused_kct_signs.inject([Float::INFINITY ,nil]) do |amax_pair,kct_sign|
        estimated_km = kct_sign.estimated_km_from_osm_kms
        @trail.osm_trail.point_with_kilometraz(estimated_km)
        amax_pair[0] > (dist = Map.distance(@trail.osm_trail.point_with_kilometraz(estimated_km),osm_guidepost.coordinates)) ? [dist,kct_sign] : amax_pair
      end
      estimated_kct_sign = max_pair[1]
      @estimated_relations[estimated_kct_sign] = [] if @estimated_relations[estimated_kct_sign].nil?
      @estimated_relations[estimated_kct_sign] << osm_guidepost
    end

  end

  def edit_with_kct
    @kct_trail =  if params[:edit_with_kctref]
      KctTrail.find_by_kctref(params[:edit_with_kctref])
    else
      @trail.kct_trail
    end
    
    unless @kct_trail
      render 'edit_with_kct_error' 
      return
    end
    
    @signs = @trail.signs.to_a
    @kct_signs = @kct_trail.kct_signs.order(:km).collect{|s| [s.km, s.kct_guidepost]}
    @km_diff = @signs.first.andand.kct_guidepost.andand.sign_on(@kct_trail).andand.km || 0
    @kct_signs_with_guidepost = @kct_signs.dup.reject{ |kg| kg[1].guidepost.nil? }.collect{|kg| kg[1]}
  end
  
  def edit_with_osm 
    @signs = @trail.signs.to_a
    @osm_signs = @trail.osm_trail.osm_signs.to_a
    @km_diff = 0
  end
  
  def update_with_kct
    (params[:kct_kilometraz] || {}).keys.each do |kct_sign_id|
      Sign.create_from_or_update( kct_kilometraz: KctKilometraz.find(kct_sign_id), trail: @trail )
    end
    
    (params[:join_to_kct_guidepost] || {}).keys.each do |key|
      kct_sign_id,kctref = key.split('|')
      kct_kilometraz = KctKilometraz.find(kct_sign_id)
      g = Guidepost.find_by_kctref(kctref).add_kctref(kct_kilometraz.kct_guidepost.kctref)
      g.save!
      Sign.create_from_or_update( kct_kilometraz: kct_kilometraz, trail: @trail )
    end
    
    redirect_to :back
  end
  
  def create
    if !trail_params[:kctref].nil? and !(@trail=Trail.find_by_kctref(trail_params[:kctref])).nil?
      respond_to do |format|
        format.html { redirect_to @trail, error: 'Trail with kctref already exists.' }
        format.json { render action: 'show', status: :error, location: @trail }
      end
    else
      @trail = Trail.new(trail_params)

      respond_to do |format|
        if @trail.save
          if @trail.initialize_color_and_type?
            @trail.save
          end
          format.html { redirect_to @trail, notice: 'Trail was successfully created.' }
          format.json { render action: 'show', status: :created, location: @trail }
        else
          format.html { render action: 'new' }
          format.json { render json: @trail.errors, status: :unprocessable_entity }
        end
      end
    end
  end

  def update
    if params[:commit]=="Zelené"
#       params[:trail][:nazev] = view_context.navrh(@trail.nazev,@trail.osm_trail.nazev) || params[:trail][:nazev]
      params[:trail][:barva] = view_context.navrh(@trail.barva,@trail.osm_trail.barva,@trail.kct_trail.barva) || params[:trail][:barva]
      params[:trail][:typ] = view_context.navrh(@trail.typ,@trail.osm_trail.typ,@trail.kct_trail.typ) || params[:trail][:typ]
    end
    if params[:commit]=="Žluté"
      params[:trail][:nazev] = view_context.navrh(@trail.osm_trail.nazev) || params[:trail][:nazev]
      params[:trail][:barva] = view_context.navrh(@trail.osm_trail.barva,@trail.kct_trail.barva) || params[:trail][:barva]
      params[:trail][:typ] = view_context.navrh(@trail.osm_trail.typ,@trail.kct_trail.typ) || params[:trail][:typ]
    end
    if params[:split]
      raise "All guideposts must have km to be able to split trail" if !@trail.signs.select{|g| g.km.blank?}.empty?
      split_params = params.require(:split).permit(:sign_id,:strana)
      split_sign = @trail.signs.find(split_params.require(:sign_id))
      raise unless ['predni','zadni'].include? split_params[:strana]
      
      if split_params[:strana]=='predni'
        new_trail = Trail.new(kctref: @trail.kctref, barva: @trail.barva, typ: @trail.typ)
        new_trail.save # aby se mu daly davat signs
        
        new_trail.signs = @trail.signs.select { |asign| asign.km < split_sign.km }
        new_trail.signs.create(guidepost_id: split_sign.guidepost_id, km: split_sign.km)
      end
      
      if new_trail.save and @trail.save
        redirect_to action: 'edit', id: new_trail.id , flash: {notice: 'Trasa byla rozdělena, toto je nově vytvořená část.'}
      else
        redirect_to :back, flash: {error: 'Chyba při rozdělování trasy, možná je to jen napůl, prověřte'} 
      end
      return 
    end
    
    respond_to do |format|
      if @trail.update_attributes(trail_params)
        format.html { redirect_to action: 'edit', flash: {notice: 'Trail was successfully updated.'} } 
        format.json { head :no_content }
      else
        format.html { redirect_to action: 'edit' }
        format.json { render json: @trail.errors, status: :unprocessable_entity }
      end
    end
  end

  def update_kilometraz
    @trail.prenes_kilometraz_z_osm!
    redirect_to :back
  end
  
  def match_guideposts
    debug = []
    if !params['match'].nil?  # pouziva se v edit_with_osm_and_kct
      params['match'].keys.collect{|x| x.split(',')}.each do |kctref,osmid| 
        debug << Sign.create_from_or_update(:kctref=>kctref, :osmid=> osmid, :trail=>@trail )
      end
    end
    if  !params['akce'].nil? # poziva se v edit_with_osm
      if !params['akce']['insert_guideposts'].nil?
        params['akce']['insert_guideposts'].each do |guidepost_id|
          debug << Sign.create_from_or_update(:guidepost=> guidepost_id, :trail=>@trail )
        end
      end
    end
    if params[:kct_guidepost] and params[:osm_guidepost]
      debug << Sign.create_from_or_update(:kctref=>params[:kct_guidepost], :osmid=> params[:osm_guidepost], :trail=>@trail )
    end    
    @trail.initialize_km; @trail.save!
    redirect_to :back
#     render :text => ERB::Util.html_escape(debug.inspect)
  end
  
  def destroy
    @trail.destroy
    respond_to do |format|
      format.html { redirect_to trails_url }
      format.json { head :no_content }
    end
  end

  def import
    raise params.inspect
  end
  
  private
  def set_trail
    begin
      if params[:id].match /kctref-(.*)/  # umožňuje použíte kctref-XXXX místo id v databázi
        @trail = Trail.find_by_kctref($1)
      else
        @trail = Trail.find(params[:id])
      end
    rescue ActiveRecord::RecordNotFound
      flash.alert = "Tato trasa není v databázi. Možná jde o starý odkaz a mezitím byla databáze reimportována."
      redirect_to :action=> :index
    end

  end
  
  def trail_params # Never trust parameters from the scary internet, only allow the white list through.
    params[:trail].andand.permit(:kctref, :osmid, :nazev, :nazev_normalized, :barva, :typ, :ok_kct)
  end
end
